//{ Semantic, Semantics } = require('./semantics.js')
//import { Semantic, Semantics } from './semantics.js'
Semantic = require('./semantics.js').Semantic;
Semantics = require('./semantics.js').Semantics;

describe('Semantic Tests', () => {
  let context;

  beforeEach( () => {
    context = {
      "amount": 10,
      "marker": "earn",
      "units": "dollars",
      "who": "joe"
    }
  });

  it('semantic match', () => {
    const matcher = ({global, context}) => {
      return context.marker == 'earn';
    }
    const handler = ({global, context}) => {}
    const semantic = new Semantic(matcher, handler);
    global = {}
    expect(semantic.matches(global, context)).toBeTruthy()
  });

  it('semantic no match', () => {
    const matcher = ({context}) => context.marker == 'earn23';
    const handler = ({global, context}) => {}
    const semantic = new Semantic(matcher, handler);
    global = {}
    expect(semantic.matches(global, context)).not.toBeTruthy()
  });

  it('semantic apply global variables', () => {
    const matcher = ({context}) => context.marker == 'earn';
    const handler = ({global, context}) => {
      global.who = context.who; 
      global.earnings = context.amount
      global.units = "dollars";
    };
    const semantic = new Semantic(matcher, handler);

    global = {}
    original = {...context}
    context_prime = semantic.apply(global, context);
    expect(context_prime).toStrictEqual(context)
    expect(context).toStrictEqual(original)
    expect(global).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
  });

  it('semantic apply context variables', () => {
    const matcher = ({context}) => context.marker == 'earn';
    const handler = ({global, context}) => {
      context.who = 'Joseph';
    };
    const semantic = new Semantic(matcher, handler);

    global = {}
    original = {...context}
    context_prime = semantic.apply(global, context);
    expect(context_prime).not.toStrictEqual(context)
    expect(context).toStrictEqual(original)
    original.who = 'Joseph'
    expect(context_prime).toStrictEqual(original)
  });
})

describe('Semantics Tests', () => {
  const contexts = [
    {
      'marker': 'earns',
      'who': 'joe',
      'earns': 10,
      'units': 'dollars',
    },
    {
      'marker': 'earns',
      'isQuery': true,
      'who': 'joe',
      'for': 'earnings'
    }
  ]

  const semantics = [
    new Semantic( 
      ({global, context}) => context.marker == 'earns' && context.isQuery, 
      ({global, context}) => { context.marker = "response"; context.earnings = global.earnings; context.units = global.units; }
    ),
    new Semantic( 
      ({global, context}) => context.marker == 'earns',
      ({global, context}) => { 
        global.who = context.who; global.earnings = context.earns; global.units = "dollars"; 
      }
    )
  ]

  it('apply one', () => {
    const s = new Semantics([semantics[1]]);
    const global = {}
    const cs = s.apply(global, [contexts[0]]);
    expect(global).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
  });

  it('apply two', () => {
    const s = new Semantics(semantics);
    const global = {}
    const cs = s.apply(global, contexts);
    expect(global).toStrictEqual({ who: 'joe', earnings: 10, units: 'dollars' });
    expect(cs[0]).toStrictEqual({ 'marker': 'earns', 'who': 'joe', 'earns': 10, 'units': 'dollars', });
    expect(cs[1]).toStrictEqual({"earnings": 10, "for": "earnings", "isQuery": true, "marker": "response", "units": "dollars", "who": "joe",})
    expect(cs).toStrictEqual([
      { 'marker': 'earns', 'who': 'joe', 'earns': 10, 'units': 'dollars', },
      {"earnings": 10, "for": "earnings", "isQuery": true, "marker": "response", "units": "dollars", "who": "joe",}
    ])
  });

  it('apply recursive', () => {
    const contexts = [ { "action": "move", "marker": "move", "thing": {"pullFromContext": true, "language": "english", "marker": "anyConcept", "word": "it"}, "place": {"id": "building1", "language": "english", "marker": "buildingConcept", "word": "building1"}}]; 
    const s= new Semantics([
            new Semantic( 
              ({global, context}) => context.pullFromContext,
              ({global, context}) => { 
                object = global.mentioned[0] 
                Object.assign(context, object) 
                delete context.pullFromContext 
              }
            )])
    const global = {"mentioned": [{"marker": "tankConcept", "word": "tank1"}]}
    const cs = s.apply(global, contexts);
    expected = [{"action": "move", "marker": "move", "place": {"id": "building1", "language": "english", "marker": "buildingConcept", "word": "building1"}, "thing": {"language": "english", "marker": "tankConcept", "word": "tank1"}}]
    expect(cs).toStrictEqual(expected)
  });
})
